@@ -1,18 +1,21 @@
 # Device::Modem - a Perl class to interface generic modems (AT-compliant)
-#
-# Copyright (C) 2002-2004 Cosimo Streppone, cosimo@cpan.org
+# Copyright (C) 2002-2007 Cosimo Streppone, cosimo@cpan.org
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the same terms as Perl itself.
 #
-# Additionally, this is ALPHA software, still needs extensive
-# testing and support for generic AT commads, so use it at your own risk,
-# and without ANY warranty! Have fun.
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# Perl licensing terms for details.
 #
-# $Id: Modem.pm,v 1.43 2004/11/10 09:20:04 cosimo Exp $
+# Commercial support is available. Write me if you are
+# interested in new features or software support.
+#
+# $Id: Modem.pm,v 1.48 2007/06/24 08:23:10 cosimo Exp $
 
 package Device::Modem;
-$VERSION = sprintf '%d.%02d', q$Revision: 1.43 $ =~ /(\d)\.(\d+)/;
+$VERSION = sprintf '%d.%02d', q$Revision: 1.48 $ =~ /(\d)\.(\d+)/;
 
 BEGIN {
 
@@ -50,17 +53,19 @@ use constant CTRL_Z => chr(26);
 use constant CR => "\r";
 
 # Connection defaults
-$Device::Modem::DEFAULT_PORT = ( $^O =~ /Win/io ) ? 'COM1' : '/dev/modem';
+$Device::Modem::DEFAULT_PORT = index($^O, 'Win') >= 0 ? 'COM1' : '/dev/modem';
 $Device::Modem::BAUDRATE = 19200;
 $Device::Modem::DATABITS = 8;
 $Device::Modem::STOPBITS = 1;
 $Device::Modem::PARITY   = 'none';
-$Device::Modem::TIMEOUT  = 500;     # milliseconds;
-$Device::Modem::WAITCYCLE= 50;
-$Device::Modem::READCHARS= 50;
+$Device::Modem::TIMEOUT  = 500;     # milliseconds
+$Device::Modem::WAITCYCLE= 20;
+$Device::Modem::READCHARS= 130;
+$Device::Modem::WAITCMD  = 20; #100;     # milliseconds
 
 # Setup text and numerical response codes
 @Device::Modem::RESPONSE = ( 'OK', undef, 'RING', 'NO CARRIER', 'ERROR', undef, 'NO DIALTONE', 'BUSY' );
+$Device::Modem::STD_RESPONSE = qr/^(OK|ERROR)$/m;
 
 #%Device::Modem::RESPONSE = (
 #	'OK'   => 'Command executed without errors',
@@ -78,7 +83,7 @@ sub new {
     my $class = ref($proto) || $proto;      # Get reference to class
 
     $aOpt{'ostype'} = $^O;                  # Store OSTYPE in object
-    $aOpt{'ostype'} =~ /Win/io and $aOpt{'ostype'} = 'windoze';
+    $aOpt{'ostype'} = 'windoze' if index( $aOpt{'ostype'}, 'Win' ) >= 0;
 
     # Initialize flags array
     $aOpt{'flags'} = {};
@@ -121,7 +126,6 @@ sub attention {
     $self->atsend('+++');
 
     # Wait 200 milliseconds
-    $self->wait(200);
     $self->answer();
 }
 
@@ -158,7 +162,8 @@ sub dial {
     }
 
     # Remove all non number chars plus some others allowed
-    $number =~ s/[^0-9,\(\)\*\-\s]//g;
+    # Thanks to Pierre Hilson for pointing out the `#' sign
+    $number =~ s/[^0-9,\(\)\*\-#\s]//g;
 
     # Dial number and wait for response
     if( length $number == 1 ) {
@@ -170,7 +175,7 @@ sub dial {
     }
 
     # XXX Check response times here (timeout!)
-    my $ans = $self->answer(undef, $timeout * 1000 );
+    my $ans = $self->answer( qr/[A-Z]/, $timeout * 1000 );
 
     if( (index($ans,'CONNECT') > -1) || (index($ans,'RING') > -1) ) {
         $ok = 1;
@@ -190,7 +195,7 @@ sub echo {
     $self->log->write( 'info', ( $lEnable ? 'enabling' : 'disabling' ) . ' echo' );
     $self->atsend( ($lEnable ? 'ATE1' : 'ATE0') . CR );
 
-    $self->answer('OK');
+    $self->answer($Device::Modem::STD_RESPONSE);
 }
 
 # Terminate current call (XXX not tested)
@@ -201,7 +206,7 @@ sub hangup {
     $self->attention();
     $self->atsend( 'ATH0' . CR );
     $self->_reset_flags();
-    $self->answer();
+    $self->answer(undef, 5000);
 }
 
 # Checks if modem is enabled (for now, it works ok for modem OFF/ON case)
@@ -274,7 +279,7 @@ sub S_register {
         $self->log->write('info', 'storing value ['.$new_value.'] into register S'.$register);
         $self->atsend( sprintf( 'AT S%02d=%d' . CR, $register, $new_value ) );
 
-        $value = ( index( $self->answer, 'OK' ) != -1 ) ? $new_value : undef;
+        $value = ( index( $self->answer(), 'OK' ) != -1 ) ? $new_value : undef;
 
     } else {
 
@@ -314,7 +319,7 @@ sub reset {
     $self->hangup();
     $self->send_init_string();
     $self->_reset_flags();
-    return $self->answer();
+#    return $self->answer();
 }
 
 # Return an hash with the status of main modem signals
@@ -350,7 +355,7 @@ sub restore_factory_settings {
     $self->log->write('warning', 'restoring factory settings '.$profile.' on '.$self->{'port'} );
     $self->atsend( 'AT&F'.$profile . CR);
 
-    $self->answer();
+    $self->answer($Device::Modem::STD_RESPONSE);
 }
 
 # Store telephone number in modem's internal address book, to dial later
@@ -392,7 +397,7 @@ sub verbose {
     $self->log->write( 'info', ( $lEnable ? 'enabling' : 'disabling' ) . ' verbose messages' );
     $self->atsend( ($lEnable ? 'ATQ0V1' : 'ATQ0V0') . CR );
 
-    $self->answer('OK');
+    $self->answer($Device::Modem::STD_RESPONSE);
 }
 
 sub wait {
@@ -432,7 +437,7 @@ sub send_init_string {
     $self->attention();
     $cInit = $self->options->{'init_string'} unless defined $cInit;
     $self->atsend('AT '.$cInit. CR );
-    $self->answer();
+    $self->answer($Device::Modem::STD_RESPONSE);
 }
 
 # returns log object reference or nothing if it is not defined
@@ -488,13 +493,23 @@ sub connect {
     # Non configurable options
     $oPort -> buffers         ( 10000, 10000 );
     $oPort -> handshake       ( 'none' );
-    $oPort -> read_const_time ( 100 );           # was 500
-    $oPort -> read_char_time  ( 10 );
+    $oPort -> read_const_time ( 20 );           # was 500
+    $oPort -> read_char_time  ( 0 );
+
+    # read_interval() seems to be unsupported on Device::SerialPort,
+    # while allowed on Win32::SerialPort...
+    if( $oPort->can('read_interval') )
+    {
+        $oPort->read_interval( 20 );
+    }
 
     $oPort -> are_match       ( 'OK' );
     $oPort -> lookclear;
 
-    $oPort -> write_settings;
+    unless ( $oPort -> write_settings ) {
+        $me->log->write( 'error', '*FAILED* write_settings on '.$me->{'port'} );
+        return $lOk;
+    }
     $oPort -> purge_all;
 
     $me-> log -> write('info', 'sending init string...' );
@@ -548,26 +563,51 @@ sub atsend {
     # Write message on port
     $me->port->purge_all();
     $cnt = $me->port->write($msg);
-    $me->wait(400);
 
-    $me->port->write_drain() unless $me->ostype eq 'windoze';
+    my $lbuf=length($msg);
+    my $ret;
+
+    while ($cnt < $lbuf)
+    {
+       $ret = $me->port->write(substr($msg, $cnt));
+       $me->write_drain();
+       last unless defined $ret;
+       $cnt += $ret;
+    }
+
     $me->log->write('debug', 'atsend: wrote '.$cnt.'/'.length($msg).' chars');
 
     # If wrote all chars of `msg', we are successful
     return $cnt == length $msg;
 }
 
+# Call write_drain() if platform allows to (no call for Win32)
+sub write_drain
+{
+    my $me = shift;
+
+    # No write_drain() call for win32 systems
+    return if $me->ostype eq 'windoze';
+
+    # No write_drain() if no port object available
+    my $port = $me->port;
+    return unless $port;
+
+    return $port->write_drain();
+}
+
 # answer() takes strings from the device until a pattern
 # is encountered or a timeout happens.
 sub _answer {
     my $me = shift;
     my($expect, $timeout) = @_;
+    $timeout = $Device::Modem::TIMEOUT if (! defined($timeout));
     my $time_slice = $Device::Modem::WAITCYCLE;     # single cycle wait time
 
     # If we expect something, we must first match against serial input
     my $done = (defined $expect and $expect ne '');
 
-$me->log->write('debug', 'answer: expecting ['.($expect||'').']'.($timeout ? ' or '.($timeout/1000).' seconds timeout' : '' ) );
+    $me->log->write('debug', 'answer: expecting ['.($expect||'').']'.($timeout ? ' or '.($timeout/1000).' seconds timeout' : '' ) );
 
     # Main read cycle
     my $cycles = 0;
@@ -577,16 +617,15 @@ $me->log->write('debug', 'answer: expecting ['.($expect||'').']'.($timeout ? ' o
     my $end_time   = 0;
 
     # If timeout was defined, check max time (timeout is in milliseconds)
-$me->log->write('debug', 'answer: timeout value is '.($timeout||'undef'));
+    $me->log->write('debug', 'answer: timeout value is '.($timeout||'undef'));
 
     if( defined $timeout && $timeout > 0 ) {
         $end_time = $start_time + ($timeout / 1000);
-
-$me->log->write( debug => 'answer: end time set to '.$end_time );
+        $end_time++ if $end_time == $start_time;
+        $me->log->write( debug => 'answer: end time set to '.$end_time );
     }
 
     do {
-
         my($howmany, $what) = $me->port->read($Device::Modem::READCHARS);
 
         # Timeout count incremented only on empty readings
@@ -597,7 +636,10 @@ $me->log->write( debug => 'answer: end time set to '.$end_time );
 
             # Check if buffer matches "expect string"
             if( defined $expect ) {
-                $done = ( defined $answer && $answer =~ $expect ) ? 1 : 0;
+                my $copy = $answer;
+                $copy =~ s/\r(\n)?/\n/g; # Convert line endings from "\r" or "\r\n" to "\n"
+                $done = ( defined $copy && $copy =~ $expect ) ? 1 : 0;
+                $me->log->write( debug => 'answer: matched expect: '.$expect ) if ($done);
             }
 
             $me->wait($time_slice) unless $done;
@@ -605,7 +647,13 @@ $me->log->write( debug => 'answer: end time set to '.$end_time );
         # Check if we reached max time for timeout (only if end_time is defined)
         } elsif( $end_time > 0 ) {
 
-            $done = time() >= $end_time ? 1 : 0;
+            $done = (time >= $end_time) ? 1 : 0;
+
+            # Read last chars in read queue
+            if( $done )
+            {
+                $me->log->write('info', 'reached timeout max wait without response');
+            }
 
         # Else we have done
         } else {
@@ -613,9 +661,9 @@ $me->log->write( debug => 'answer: end time set to '.$end_time );
             $done = 1;
         }
 
-$me->log->write('debug', 'done='.$done.' end_time='.$end_time.' now='.time().' start_time='.$start_time );
+        $me->log->write('debug', 'done='.$done.' end='.$end_time.' now='.time().' start='.$start_time );
 
-      } while (not $done);
+    } while (not $done);
 
     $me->log->write('info', 'answer: read ['.($answer||'').']' );
 
@@ -1292,8 +1340,10 @@ Waits (yea) for a given amount of time (in milliseconds). Usage:
 Wait is implemented via C<select> system call.
 Don't know if this is really a problem on some platforms.
 
+=head2 write_drain()
 
-
+Only a simple wrapper around C<Device::SerialPort::write_drain> method.
+Disabled for Win32 platform, that doesn't have that.
 
 
 =head1 REQUIRES
@@ -1352,7 +1402,7 @@ Cosimo Streppone, L<cosimo@cpan.org>
 
 =head1 COPYRIGHT
 
-(C) 2002-2004 Cosimo Streppone, L<cosimo@cpan.org>
+(C) 2002-2007 Cosimo Streppone, L<cosimo@cpan.org>
 
 This library is free software; you can only redistribute it and/or
 modify it under the same terms as Perl itself.
